"""規模の小さいアプリケーションでたまに許されてる.iniのセクション被り対応版ini controller"""
# region	|=-=-=-=|Import  Section|=-=-=-=|
from collections import namedtuple
from typing import (
    NamedTuple,
    overload,
    Literal,
    TextIO,
    AnyStr,
    Any,
    IO,
)
import __init__
from types import ModuleType
# endregion
# 			|=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=|

# region	|=-=-=-=|Setup Section|=-=-=-=|
__all__ = [
    'Encoding',
    'INIsNotFound',
    'INI',
    'AbsTrue',
    'AbsFalse',
    'AbsTrueFalse',
    'StrBool',
    'str_bool'
]
ini_item_string = type('ini_format_string', (str,))
ini_section_string = type('ini_format_string', (str,))


class Encoding(NamedTuple):
    SJIS: str
    UTF8: str


Encoding = Encoding(...)
# endregion
# 			|=-=-=-=-=-=-=-=-=-=-=-=-=-=-=|


# iniファイルない時用の例外
class INIsNotFound(Exception):
    '''Required :ref:`.ini` Files Does Not Exist'''
    # .iniファイル必要なのに無いやで。ってやつ

    # classに直接文字列要求(例: str(), print(), raise)があったときに自動で処理される特殊メソッド
    def __str__(self) -> str: ...


# 大本営
class INI:
    def __init__(self, filepath: str = None, encoding: str = 'shift-jis') -> None:
        """.iniファイル編集するのに便利系クラス\n
        Parameters:
            filepath (str): 読み込みたい.iniのフルパス。`None`で`raise`\n
            encoding (str): 読み込むファイルのエンコード方式。現状、書き出しと共通設定。未指定時 -> `shift-jis`"""
        if filepath is not None:
            self.file_path: str
            self.out_file: str
        self.chrdec: str
        self._context: bool
        self.ini_content: dict
        self._already_exist: dict
        self._exist_lower: dict

    def _keys(self, ini_contents: dict = ...) -> list:
        """dictの第一keyをlistで返すやつ\n
        Examples:
            >>> fruit_dict = \
{'apple': 'red', 'banana': 'yellow', 'orange': 'orange'}
            >>> _sections(fruit_dict)
            ['apple', 'banana', 'orange']"""
        ...

    def _ini_item(self, item: str = ..., value: str = ...) -> ini_item_string:
        """itemとvalueを'='でつないで.iniフォーマットにして返す\n
        Example:
            >>> _ini_item('apple', 'red')
            'apple=red'"""
        ...

    def _item(self, sect, items: tuple[str, Any] = ...) -> None: ...
    def _section(self, name: str = ...) -> None: ...

    def _section_wrapp(self, key: str) -> ini_section_string:
        """dictの第一key(セクション名)を.iniのセクション用にラップするやつ\n
        Example:
            >>> _section_wrapp('section')
                -> '[section]'"""
        ...

    def _write_section(self, section: str = ..., wrapp: bool = False) -> str | ini_section_string:
        """ダブりセクションがあったときに一時的にリネームして保持するから、
        それを本来のセクション名に戻して返すメソッド。ダブってなかったらArg=return\n
        wrapp=Trueで_section_wrapp()も一緒に処理して返す。
        Example:
            example.ini:
                [section]\n
                apple=red\n
                banana=yellow\n
                ...\n
                [section]\n
                orange=orange\n
                [anysection]\n
                ...\n
            >>> ini = \
{'section': {'apple': 'red', 'banana': 'yellow', ...},\\\n
            >>>        'section1': {'orange': 'orange'},\\\n
            >>>        'anysection': ...}
            >>> _write_section('section1')
                -> 'section'
            >>> _writesection('anysection', wrapp=True)
                -> '[anysection]'"""
        ...

    def config(self, OutputPath: str = None) -> None: ...

    def export(self) -> TextIO:
        """configで設定した出力ファイルに.iniフォーマットで編集した内容を書き出す。
        出力ファイルを設定してなかったら元ファイルを上書き"""
        ...

    def isexist(self, key: str) -> str:
        """### :code:`key in self._already_exist` ⇓ \n
        Returns:
            renamed key

        ### :code:`key not in self._already_exist` ⇓ \n
        Returns:
            input to arg key"""
        ...

    def read(self, ini_file: IO = None) -> dict[dict[AnyStr, ...], ...] | Exception: ...
    @overload
    def __getitem__(self, key: tuple[str, str]) -> dict | str | None: ...
    @overload
    def __getitem__(self, key: str = '*') -> dict[dict, ...]: ...
    @overload
    def __getitem__(self, key: str = AnyStr) -> None | dict: ...
    @overload
    def __getitem__(self, key=None) -> None: ...
    def __setitem__(self, key: tuple[str, str], value) -> None: ...
    def __enter__(self) -> 'ini':
        self._context = True
        self.file = open(self.file_path, encoding=self.chrdec)

        return self

    def __exit__(self, exc_type, exc_val, exc_tb) -> None:
        self.file.close()


class _StrOrBool:
    def __init__(self, content: '_StrOrBool'):
        self.content = content

    def STRING(self, content: '_StrOrBool' = None):
        ...

    def BOOL(self, content: '_StrOrBool' = None):
        ...

    def __str__(self):
        """
        Example:
            _SubStrOrBool_instance << str('true')
            >>> print(str(_SubStrOrBool_instance))
            bool(True)
        """
        return self.STRING(self.content)

    def __init_true__(self):
        class _true:
            def __init__(subself):
                subself.trues = (True, 1, 'true', '1')

            @property
            def true(subself):
                return subself.trues

            def __contain__(subself, item: str | bool | int):
                if isinstance(item, str):
                    return item.lower() in subself.trues
                else:
                    return item in subself.trues

        return _true()

    def __init_false__(self):
        class _false:
            def __init__(subself):
                subself.falses = (False, 0, 'false', '0', None)

            @property
            def false(subself):
                return subself.falses

            def __contain__(subself, item: str | bool | int):
                if isinstance(item, str):
                    return item.lower() in subself.falses
                else:
                    return item in subself.falses

        return _false()

    def __init_subclass__(cls, *args, **kwargs) -> None:
        super().__init_subclass__(*args, **kwargs)
        cls.true = cls.__init_true__(cls)
        cls.false = cls.__init_false__(cls)

    @property
    def trues(self):
        return self.__init_true__().trues

    @property
    def falses(self):
        return self.__init_false__().falses


class AbsTrue(_StrOrBool):
    def __init__(self, content: _StrOrBool):
        super().__init__(content)

    def __bool__(self) -> bool:
        return self.content in self.true


class AbsFalse(_StrOrBool):
    def __init__(self, content: _StrOrBool):
        super().__init__(content)

    def __bool__(self) -> bool:
        return self.content in self.false


class AbsTrueFalse(_StrOrBool):
    def __init__(self, content: _StrOrBool):
        super().__init__(content)

    def __bool__(self) -> bool:
        if self.content in self.true:
            return True
        elif self.content in self.false:
            return False
        else:
            return None


class StrBool:
    def __new__(cls, content: _StrOrBool, absolute: bool = None) -> AbsTrue | AbsFalse | AbsTrueFalse: ...


def str_bool(content: str):
    trues = (True, 1, 'true', '1')
    falses = (False, 0, 'false', '0', None)

    if content.lower() in trues:
        return True
    elif content.lower() in falses:
        return False
    else:
        return None



import tkinter as tk
from typing import Any, Callable, Iterable, Literal, Self
from typing_extensions import override
AUTO = -1

type ScreenUnits = str | float
type Cursor = str | tuple[str] | tuple[str, str] | tuple[str, str, str] | tuple[str, str, str, str]
type Relief = Literal["raised", "sunken", "flat", "ridge", "solid", "groove"]
type TakeFocusValue = bool | Literal[0, 1, ""] | Callable[[str], bool | None]
def signeture[**Arg, Ret](f: Callable[Arg, Any]) -> Callable[[Callable[..., Ret]], Callable[Arg, Ret]]: ...

class Transition(tk.Frame):
    def __init__(self,
                 master: tk.Misc | None = None,
                 cnf: dict[str, Any] | None = {},
                 *,
                 loop: bool = False,
                 background: str = ...,
                 bd: ScreenUnits = 0,
                 bg: str = ...,
                 border: ScreenUnits = 0,
                 borderwidth: ScreenUnits = 0,
                 class_: str = "Frame",
                 colormap: tk.Misc | Literal['new', ''] = "",
                 container: bool = False,
                 cursor: Cursor = "",
                 height: ScreenUnits = 0,
                 highlightbackground: str = ...,
                 highlightcolor: str = ...,
                 highlightthickness: ScreenUnits = 0,
                 name: str = ...,
                 padx: ScreenUnits = 0,
                 pady: ScreenUnits = 0,
                 relief: Relief = "flat",
                 takefocus: TakeFocusValue = 0,
                 visual: str | tuple[str, int] = "",
                 width: ScreenUnits = 0) -> None: ...

    @override
    def __getitem__(self, id: int) -> tk.Widget: ...
    def __add__(self, value: tk.Widget | Iterable[tk.Widget]) -> Transition: ...
    def __iadd__(self, value: tk.Widget | Iterable[tk.Widget]) -> Self: ...
    def register_widget(self, widget: tk.Widget | Iterable[tk.Widget], *, id: int | slice = AUTO) -> None:
        """遷移するためのウィジェット登録メソッド
        idは__getitem__で取得できるようにするためだけの識別子
        既存のidを指定してregisterした場合は上書き。存在しないidを指定してregisterした場合はIndexError。
        .next(), .prev()の表示順は登録順"""
        ...
    @signeture(tk.Widget.pack)
    def prepack(self, *arg: Any, **kw: Any) -> None:
        """.next(), .prev()でパックするときのオプションをあらかじめセットしておくやつ"""
        ...
    def initial_pack(self, id: int = 0) -> None:
        "prepack()で指定したオプションに従って、引数で指定したidのウィジェットをあらかじめ配置してくれる"
        ...
    type isLastPage = bool
    def next(self) -> isLastPage:
        """今配置されてるウィジェットの次のIDのウィジェットを配置する
        loop = Falseのとき、今配置されてるウィジェットIDより後ろが無い場合return True
        それ以外はreturn False"""
        ...
    type isFirstPage = bool
    def prev(self) -> isFirstPage:
        """今配置されてるウィジェットの前のIDのウィジェットを配置する
        loop = Falseのとき、今配置されているウィジェットIDより前が無い場合return True
        それ以外はreturn False"""
        ...

def sample(loop: bool = True, widget_number: int = 3) -> None:
    """~~~markdown

    __Transition__ クラスがどんな感じに使えるかを見るサンプル
    以下関数内コード
    ======================================================
    ~~~

    ~~~python
    root = tk.Tk()
    root.geometry('250x250')
    trans = Transition(root, loop=loop)
    trans.pack(side=tk.TOP, anchor=tk.S, fill=tk.BOTH, expand=True)
    trans.prepack(fill=tk.BOTH, expand=True)

    for i in range(1, widget_number + 1):
        _tmp_ = tk.Frame(trans)
        tk.Label(_tmp_, text=f'LABEL{i}', font=font.Font(size=20)).pack(fill=tk.BOTH, expand=True)
        trans.register_widget(_tmp_)

    bf = tk.Frame(root)
    bf.pack(side=tk.TOP, anchor=tk.S, fill=tk.X, expand=True)
    tk.Button(bf, text='NEXT', command=trans.next).pack(side=tk.RIGHT, anchor=tk.E)
    tk.Button(bf, text='PREV', command=trans.prev).pack(side=tk.RIGHT, anchor=tk.E, padx=5)
    trans.initial_pack()
    root.mainloop()"""



class ModeSelectionError(Exception):
    """Error when there is a mode that does not exist in the 'mode=' arguments"""
    def __str__(self):
        return 'Selecting a Mode That Doesn\'t Exist'
    
def modeselectionerror():
    raise ModeSelectionError


from ..e_typing import (
    ColorStdOut,
)
from functools import wraps
from .cprint import (
    Color,
)
import sys
__all__ = [
    'DebugPrint',
    'log',
]


def log(cfunc=None, /,
        *, debugging=True,
        color=Color.blue,
        show_caller=True,
        show_called=True,
        show_args=True,
        show_returns=True):
    match cfunc:
        case None:
            def decorator(func, /):
                if not debugging: return func
                @wraps(func)
                def wrapper(*args, **kwargs):
                    show = ''
                    flg = False
                    if show_caller:
                        c_frame = sys._getframe(1)
                        _mid = f'{c_frame.f_globals["__name__"]}.'
                        _mid += f'{c_frame.f_code.co_name}'
                        show += f'Caller: {_mid}'
                        del _mid
                        flg = True
                    if show_called:
                        if flg:
                            show += '\n'
                        show += f'Called: {func.__name__}'
                        flg = True
                    if show_args:
                        if flg:
                            show += '\n'
                        show += f'Args: {args}, {kwargs}'
                        flg = True
                    if flg:
                        color(show)
                        del show, flg
                    result = func(*args, **kwargs)
                    if show_returns:
                        color(f'Returns: {result}')
                    return result
                return wrapper
            return decorator
        case f:
            @wraps(f)
            def decorator(func):
                if not debugging: return func
                @wraps(func)
                def wrapper(*args, **kwargs):
                    show = ''
                    flg = False
                    if show_caller:
                        c_frame = sys._getframe(1)
                        _mid = f'{c_frame.f_globals["__name__"]}.'
                        _mid += f'{c_frame.f_code.co_name}'
                        show += f'Caller: {_mid}'
                        del _mid
                        flg = True
                    if show_called:
                        if flg:
                            show += '\n'
                        show += f'Called: {func.__name__}'
                        flg = True
                    if show_args:
                        if flg:
                            show += '\n'
                        show += f'Args: {args}, {kwargs}'
                        flg = True
                    if flg:
                        color(show)
                        del show, flg
                    result = func(*args, **kwargs)
                    if show_returns:
                        color(f'Returns: {result}')
                    return result
                return wrapper
            return decorator(f)


class DebugPrint:
    __slots__ = ('debug', 'nl_num')

    def __init__(self, debug_flg, newline_number=3):
        """デバッグ用ログをデバッグフラグがTrueならcall構文で指定した引数の内容を色違いでコンソール出力してくれるクラス\n
        出力する項目をnewline_number個ごとに改行して出力を見やすくする"""
        self.debug = debug_flg
        self.nl_num = newline_number

    def __call__(self, *args):
        """examples:
            >>> debug = DebugPrint(True)
            >>> debug('debug', 'to', 'example')
            (light green)debug: debug, to, example
            >>> debug = DebugPrint(True, newline_number = 2)
            >>> debug('debug', 'to', 'example')
            debug: debug, to,
            \texample"""
        nt = '\n\t'
        spc = ' '
        if self.debug:
            for i in range(len(args)):
                if i == 0:
                    content = args[i]
                else:
                    mid_entry = nt if i % self.nl_num == 0 else spc
                    content = f'{content},{mid_entry}{args[i]}'

            print(f'\033[92mdebug: {content}\033[0m')



"SQLAlchemyのメモリ生成データベース"
from typing import Any, Callable, Literal, overload
from sqlalchemy import orm
from sqlalchemy.orm.decl_api import DeclarativeMeta
from sqlalchemy import Engine, create_engine
from sqlalchemy.pool import (
    ConnectionPoolEntry,
    Pool,
)
from sqlalchemy.pool.base import ResetStyle
from sqlalchemy.util import (
    immutabledict,
)
from sqlalchemy.engine.interfaces import (
    IsolationLevel,
    DBAPIConnection,
)
Base: DeclarativeMeta
"""```python

# usage:
class User(Base): ...
class Tweet(Base): ...

user = User(...)
tweet = Tweet(...)

engine = ...
Base.metadata.create_all(engine)
...
```"""

def _info[Class](cls: Class) -> Callable[[Callable[..., Any]], Class]: ...

@_info(orm.sessionmaker)
class sessionmaker: ...


@overload
def memory_engine(
    *,
    connect_args: dict[Any, Any] = ...,
    convert_unicode: bool = ...,
    creator: Callable[..., DBAPIConnection] | Callable[[ConnectionPoolEntry], DBAPIConnection] = ...,
    echo: None | bool | Literal['debug'] = ...,
    echo_pool: None | bool | Literal['debug'] = ...,
    enable_from_linting: bool = ...,
    execution_options: immutabledict[str, Any] = ...,
    future: Literal[True],
    hide_parameters: bool = ...,
    implicit_returning: Literal[True] = ...,
    insertmanyvalues_page_size: int = ...,
    isolation_level: IsolationLevel = ...,
    json_deserializer: Callable[..., Any] = ...,
    json_serializer: Callable[..., Any] = ...,
    label_length: int | None = ...,
    logging_name: str = ...,
    max_identifier_length: int | None = ...,
    max_overflow: int = ...,
    module: Any | None = ...,
    paramstyle: Literal["qmark", "numeric", "named", "format", "pyformat", "numeric_dollar"] | None = ...,
    pool: Pool | None = ...,
    poolclass: type[Pool] | None = ...,
    pool_logging_name: str = ...,
    pool_pre_ping: bool = ...,
    pool_size: int = ...,
    pool_recycle: int = ...,
    pool_reset_on_return: ResetStyle | bool | Literal['commit', 'rollback'] | None = ...,
    pool_timeout: float = ...,
    pool_use_lifo: bool = ...,
    plugins: list[str] = ...,
    query_cache_size: int = ...,
    use_insertmanyvalues: bool = ...,
    **kwargs: Any
) -> Engine:
    """```python

    # definition:
    memory_engine = partial(sqlalchemy.create_engine, 'sqlite:///:memory:')

    # usage:
    engine = memory_engine(echo=True)
    Base.metadata.create_all(engine)
    Session = sessionmaker(bind=engine)
    session = Session()
    ```"""


@overload
def memory_engine(**kwargs: Any) -> Engine:
    """```python

    # definition:
    memory_engine = partial(sqlalchemy.create_engine, 'sqlite:///:memory:')

    # usage:
    engine = memory_engine(echo=True)
    Base.metadata.create_all(engine)
    Session = sessionmaker(bind=engine)
    session = Session()
    ```"""


def make_meta(base: DeclarativeMeta = ..., engine: Engine = ...) -> None:
    """Base.metadata.create_all(engine)の代行サービス"""



from ..e_typing import (
    ColorStdOut,
    CanStdOut,
    StdOut,
)
from functools import wraps, _Wrapper
from typing import (
    overload,
    ParamSpec,
    Callable,
    TypeVar,
    TypeVarTuple,
    Any,
)
from .cprint import (
    Color,
)
__all__ = [
    'DebugPrint',
    'log',
]
_Args = ParamSpec('_Args')
_Returns = TypeVar('_Returns')
type Wrapped[**Par, Ret] = Callable[[Callable[Par, Ret]], Callable[Par, Ret]]


def log[**Args, Returns](cfunc: Callable[Args, Returns] | None = ..., /,
        *, debugging: bool = True,
        color: Color.colors = Color.blue,
        show_caller: bool = True,
        show_called: bool = True,
        show_args: bool = True,
        show_returns: bool = True) -> Wrapped[Args, Returns]: ...


class DebugPrint:
    __slots__ = ('debug', 'nl_num')

    def __init__(self, debug_flg: bool, newline_number: int = 3) -> None:
        """デバッグ用ログをデバッグフラグがTrueならcall構文で指定した引数の内容を色違いでコンソール出力してくれるクラス\n
        出力する項目をnewline_number個ごとに改行して出力を見やすくする"""
        self.debug = debug_flg
        self.nl_num = newline_number

    def __call__[*args](self, *args: *args) -> ColorStdOut[*args]:
        """examples:
            >>> debug = DebugPrint(True)
            >>> debug('debug', 'to', 'example')
            (light green)debug: debug, to, example
            >>> debug = DebugPrint(True, newline_number = 2)
            >>> debug('debug', 'to', 'example')
            debug: debug, to,
            \texample"""
        ...



from __future__ import annotations
from dataclasses import dataclass
from ..e_typing import (
    ColorStdOut,
)
from typing import (
    Callable,
    TypeVar,
    TypeVarTuple,
    TypeAlias,
    Generic,
    Literal,
    Type,
    Any,
)
__all__ = [
    # const
    'Color',

    # functions
    # #custom
    'color_print',

    # #presets
    'blue',
    'gray',
    'green',
    'magenta',
    'red',
    'turquoise',
    'yellow',

    # #decoration
    'border',
]
_AT = TypeVar('_AT', bound=Callable[..., Any])
values = TypeVarTuple('values')
default = TypeVar('default', Literal[True], Literal[False])


def color_print(*args: _AT,
                color: str,
                undo_when_exit: bool = True) -> ColorStdOut[_AT]:
    "Print to Console in `Color of Custom`"
    ...


def gray(*values: _AT, default: bool = True) -> ColorStdOut[_AT]:
    "Print to Console in `Gray Color`"
    ...


def red(*values: _AT, default: bool = True) -> ColorStdOut[_AT]:
    "Print to Console in `Red Color`"
    ...


def green(*values: _AT, default: bool = True) -> ColorStdOut[_AT]:
    "Print to Console in `Green Color`"
    ...


def yellow(*values: _AT, default: bool = True) -> ColorStdOut[_AT]:
    "Print to Console in `Yellow Color`"
    ...


def blue(*values: _AT, default: bool = True) -> ColorStdOut[_AT]:
    "Print to Console in `Blue Color`"
    ...


def magenta(*values: _AT, default: bool = True) -> ColorStdOut[_AT]:
    "Print to Console in `Magenta Color`"
    ...


def turquoise(*values: _AT, default: bool = True) -> ColorStdOut[_AT]:
    "Print to Console in `Turquoise Color`"
    ...


def border(*values: _AT, default: bool = True) -> ColorStdOut[_AT]:
    "Print to Console in `Border String`"
    ...


@dataclass(frozen=True)
class Color:
    """```Python

class Color:
    def blue(*values: _AT, default: bool = True) -> ColorStdOut[_AT]: ...
    def gray(*values: _AT, default: bool = True) -> ColorStdOut[_AT]: ...
    def green(*values: _AT, default: bool = True) -> ColorStdOut[_AT]: ...
    def magenta(*values: _AT, default: bool = True) -> ColorStdOut[_AT]: ...
    def red(*values: _AT, default: bool = True) -> ColorStdOut[_AT]: ...
    def turquoise(*values: _AT, default: bool = True) -> ColorStdOut[_AT]: ...
    def yellow(*values: _AT, default: bool = True) -> ColorStdOut[_AT]: ...
    def border(*values: _AT, default: bool = True) -> ColorStdOut[_AT]: ...
```
```MarkDown
======================================================
Constant of `cprint` Functions
```"""
    colors: Type[Callable[[*values, default], ColorStdOut[*values]]]
    blue: Callable[[*values, default], ColorStdOut[*values]]
    gray: Callable[[*values, default], ColorStdOut[*values]]
    green: Callable[[*values, default], ColorStdOut[*values]]
    magenta: Callable[[*values, default], ColorStdOut[*values]]
    red: Callable[[*values, default], ColorStdOut[*values]]
    turquoise: Callable[[*values, default], ColorStdOut[*values]]
    yellow: Callable[[*values, default], ColorStdOut[*values]]
    border: Callable[[*values, default], ColorStdOut[*values]]



"SQLAlchemyのメモリ生成データベース"
from functools import partial
from sqlalchemy import Boolean, Engine, LargeBinary, create_engine, Column, Integer, String, ForeignKey
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.orm import sessionmaker, relationship, Mapped, backref, session as _session
from sqlalchemy.orm.decl_api import DeclarativeMeta

Base = declarative_base()
"""```python

# usage:
class User(Base): ...
class Tweet(Base): ...

user = User(...)
tweet = Tweet(...)

engine = ...
Base.metadata.create_all(engine)
...
```"""

memory_engine = partial(create_engine, 'sqlite:///:memory:')
"""```python

# definition:
memory_engine = partial(sqlalchemy.create_engine, 'sqlite:///:memory:')

# usage:
engine = memory_engine(echo=True)
Base.metadata.create_all(engine)
Session = sessionmaker(bind=engine)
session = Session()
```"""


def make_meta(base, engine):
    """Base.metadata.create_all(engine)の代行サービス"""
    base.metadata.create_all(engine)



"""It provides an extra class that has no function and can only be used for type hints.\n
何の機能も無い、型ヒントだけに使えるエクストラなクラスを提供するやで。"""
import dataclasses
from typing import (
    TypeVarTuple,
    Callable,
    Generic,
    TypeVar,
    Any,
)
__all__ = [
    # Original Types
    'ColorStdOut',
    'NotIterable',
    'StdOut',
    'SupportIndex',
    'SupportPath',

    # Standard Lib
    # # dataclasses
    'DataClass',
]

_AT = TypeVar('_AT')  # T は任意の型を表す型変数
_ET = TypeVar('_ET')
_ATs = TypeVarTuple('_ATs')
_F = TypeVar('_F', bound=Callable[..., Any])


# Original Types
class Function(Callable[_F, _AT]):
    def __getitem__(self, key: _F) -> _AT:
        ...


class NotIterable:
    """Mixin to prevent iteration, without being compatible with Iterable.

    That is, we could do:
        def __iter__(self): raise TypeError()
    But this would make users of this mixin duck type-compatible with
    collections.abc.Iterable - isinstance(foo, Iterable) would be True.

    Luckily, we can instead prevent iteration by setting __iter__ to None, which
    is treated specially.
    """

    __slots__ = ()
    __iter__ = None


class StdOut(Generic[*_ATs]):
    '<TypeHints to indicate that it will be printed to the console>'
    def __str__(self) -> str:
        return '<type hints StdOut>'

    def __repr__(self) -> str:
        return self.__doc__


class ColorStdOut(StdOut[*_ATs], Generic[*_ATs]):
    def __str__(self) -> str:
        return '<type hints ColorStdOut>'


class CanStdOut(Generic[*_ATs]):
    ...


class SupportIndex(Generic[*_ATs]):
    ...


class SupportPath(Generic[*_ATs]):
    ...


# Types related to classes and functions of stdlib
# # dataclasses
class _DataclassParams(dataclasses._DataclassParams):
    def __init__(self, init, repr, eq, order, unsafe_hash, frozen) -> None:
        self.init: bool
        self.repr: bool
        self.eq: bool
        self.order: bool
        self.unsafe_hash: bool
        self.frozen: bool
    def __repr__(self) -> str: ...
class DataClass:
    __dataclass_fields__: dict[str, dataclasses.Field]
    __dataclass_params__: _DataclassParams
    def __post_init__(self) -> None: ...



class NotBoolStrings(Exception):
    """Exception that occurs at the time of A"""
    def __str__(self):
        return 'Not BoolStrings Error'

def strbool(TargetString: str) -> bool:
    lowstr = TargetString.lower()
    if lowstr == 'true':
        return True
    elif lowstr == 'false':
        return False
    else:
        try:
            raise NotBoolStrings
        except NotBoolStrings as e:
            print('Set Arguments isn\'t BoolStrings') 
    
if __name__ == '__main__':
    strbool('tru')


"""
SQL databases controler
"""
from sqlalchemy import BinaryExpression, ColumnElement, Engine as _Engine
from sqlalchemy.orm import Session as _Session
from typing import TYPE_CHECKING as _TC
if _TC:
    from sqlalchemy.inspection import Inspectable
    from sqlalchemy.orm.util import AliasedClass, AliasedInsp, Mapper
    from sqlalchemy.sql.elements import SQLCoreOperations
    from sqlalchemy.sql.roles import TypedColumnsClauseRole, ColumnsClauseRole, ExpressionElementRole
    from typing import Any, Generator, Literal, Protocol
    class HasClauseElement[TYPE](Protocol):
        """indicates a class that has a __clause_element__() method"""

        def __clause_element__(self) -> ExpressionElementRole[TYPE]:
            ...
    type EntityType[TYPE] = type[TYPE] | AliasedClass[TYPE] | Mapper[TYPE] | AliasedInsp[TYPE]
    type TypedColumnClauseArgument[TYPE] = TypedColumnsClauseRole[TYPE] | SQLCoreOperations[TYPE] | type[TYPE]
    type ColumnsClauseArgument[TYPE] = (TypedColumnsClauseRole[TYPE]
                                        | ColumnsClauseRole
                                        | SQLCoreOperations[TYPE]
                                        | Literal["*", 1]
                                        | type[TYPE]
                                        | Inspectable[HasClauseElement[TYPE]]
                                        | HasClauseElement[TYPE])


def items(url_or_engine: str | _Engine = 'sqlite:///:memory:', *, print_to_console: bool = True, **kw: Any) -> dict[str, list[Any]] | dict[None, None]: ...

def query_gen[TYPE](session: _Session,
                    table: (EntityType[TYPE]
                            | TypedColumnsClauseRole[TYPE]
                            | TypedColumnClauseArgument[TYPE]
                            | ColumnsClauseArgument[TYPE]
                            | None) = None,
                    *filters: ColumnElement[bool] | BinaryExpression[bool],
                    delete: bool = False, commit: bool = False) -> Generator[TYPE, None, None]: ...



"""It provides an extra class that has no function and can only be used for type hints.\n
何の機能も無い、型ヒントだけに使えるエクストラなクラスを提供するやで。"""
import dataclasses
from typing import (
    TypeVarTuple,
    ParamSpec,
    Callable,
    Sequence,
    Generic,
    TypeVar,
    Any,
)
__all__ = [
    # Original Types
    'ColorStdOut',
    'NotIterable',
    'StdOut',
    'SupportIndex',
    'SupportPath',

    # Standard Lib
    # # dataclasses
    'DataClass',
]


_AT = TypeVar('_AT')  # T は任意の型を表す型変数
_ATs = TypeVarTuple('_ATs')
_AT_co = TypeVar('_AT_co', covariant=True)


values = TypeVarTuple('values')
default = TypeVar('default', bound=bool)
a: Callable[[*values, default], str]


arg_names = TypeVarTuple('arg_names')
return_type = TypeVar('return_type')


# Original Types
""" class Function(Callable[[*arg_names], return_type]):
    "developing now"
    ... """


class NotIterable:
    """Mixin to prevent iteration, without being compatible with Iterable.
    Iterable との互換性がなく、繰り返しを防止するためにミックスインします。

    That is, we could do:
    つまり、次のことができます:
        def __iter__(self): raise TypeError()
    But this would make users of this mixin duck type-compatible with
    collections.abc.Iterable - isinstance(foo, Iterable) would be True.

    Luckily, we can instead prevent iteration by setting __iter__ to None,
    which is treated specially.
    """

    __slots__ = ()
    __iter__ = None


class StdOut(Generic[*_ATs]):
    'TypeHints to indicate that it will be printed to the console'

    def __str__(self) -> str:
        return '<type hints StdOut>'

    def __repr__(self) -> str:
        return f'<{self.__doc__}>'


# class ColorStdOut(StdOut[*_ATs], Generic[*_ATs]):
class ColorStdOut(StdOut[*_ATs]):
    'TypeHints to indicate that it will be printed of ColorString to the console'

    def __str__(self) -> str:
        return '<type hints ColorStdOut>'


class CanStdOut(Generic[*_ATs]):
    ...


class SupportIndex(Generic[*_ATs]):
    ...


class SupportPath(Generic[*_ATs]):
    ...


# Types related to classes and functions of stdlib
# # dataclasses
class _DataclassParams:
    __slots__ = ('init',
                 'repr',
                 'eq',
                 'order',
                 'unsafe_hash',
                 'frozen',)

    def __init__(self,
                 init: bool = True,
                 repr: bool = True,
                 eq: bool = True,
                 order: bool = False,
                 unsafe_hash: bool = False,
                 frozen: bool = False):
        self.init = init
        self.repr = repr
        self.eq = eq
        self.order = order
        self.unsafe_hash = unsafe_hash
        self.frozen = frozen

    def __repr__(self):
        return ('_DataclassParams('
                f'init={self.init!r},'
                f'repr={self.repr!r},'
                f'eq={self.eq!r},'
                f'order={self.order!r},'
                f'unsafe_hash={self.unsafe_hash!r},'
                f'frozen={self.frozen!r}'
                ')')


class DataClass:
    __dataclass_fields__: dict[str, dataclasses.Field[Any]]
    __dataclass_params__: _DataclassParams

    def __post_init__(self) -> None:
        pass



from ..e_typing import (
    ColorStdOut,
    NotIterable
)
from typing import (
    TypeVar,
)
__all__ = [
    # const
    'Color',

    # functions
    # #custom
    'color_print',

    # #presets
    'blue',
    'gray',
    'green',
    'magenta',
    'red',
    'turquoise',
    'yellow',

    # #decoration
    'border',
]
# Any Type
_AT = TypeVar('_AT')


def color_print(*args: _AT,
                color: str,
                undo_when_exit: bool = True) -> ColorStdOut[_AT]:
    "Print to Console in `Color of Custom`"
    undo = '\033[0m\n' if undo_when_exit else '\n'
    print(color, end='')
    print(*args, end=undo)


def gray(*values: _AT, default: bool = True) -> ColorStdOut[_AT]:
    "Print to Console in `Gray Color`"
    color_print(*values, color='\033[90m', undo_when_exit=default)


def red(*values: _AT, default: bool = True) -> ColorStdOut[_AT]:
    "Print to Console in `Red Color`"
    color_print(*values, color='\033[91m', undo_when_exit=default)


def green(*values: _AT, default: bool = True) -> ColorStdOut[_AT]:
    "Print to Console in `Green Color`"
    color_print(*values, color='\033[92m', undo_when_exit=default)


def yellow(*values: _AT, default: bool = True) -> ColorStdOut[_AT]:
    "Print to Console in `Yellow Color`"
    color_print(*values, color='\033[93m', undo_when_exit=default)


def blue(*values: _AT, default: bool = True) -> ColorStdOut[_AT]:
    "Print to Console in `Blue Color`"
    color_print(*values, color='\033[94m', undo_when_exit=default)


def magenta(*values: _AT, default: bool = True) -> ColorStdOut[_AT]:
    "Print to Console in `Magenta Color`"
    color_print(*values, color='\033[95m', undo_when_exit=default)


def turquoise(*values: _AT, default: bool = True) -> ColorStdOut[_AT]:
    "Print to Console in `Turquoise Color`"
    color_print(*values, color='\033[96m', undo_when_exit=default)


def border(*values: _AT, default: bool = True) -> ColorStdOut[_AT]:
    "Print to Console in `Border String`"
    color_print(*values, color='\033[97m', undo_when_exit=default)


class Color(NotIterable):
    "Constant of 'cprint' Function"
    colors = type('colors', (), {})
    blue = blue
    gray = gray
    green = green
    magenta = magenta
    red = red
    turquoise = turquoise
    yellow = yellow
    border = border


if __name__ == '__main__':
    blue('test', 'to', 'debug')



import os
import logging
import datetime as dt
from typing import NewType
from dataclasses import dataclass

__all__ = [
    'Log_Out',
    'LEVEL',
    'MODE',
]

# |=-=-=-=|Setup Section|=-=-=-=|
# alias
op = os.path

# type
level = NewType('level', int)
mode = NewType('mode', str)

# initialize valiable
_PARENT_MODULE = op.basename(op.abspath(__file__))
_PARENT_NAME = op.splitext(_PARENT_MODULE)[0]
_DEFAULT_PATH = r'E:\DevelopmentEnvironment\python\logs'
_DEFAULT_LOGNAME = (
    f'{_PARENT_MODULE}[{dt.datetime.now():%Y%m%d%H%M%S}].log'
)
_ERROR_LOG_DIR = r'E:\DevelopmentEnvironment\ERROR LOG\EXCEPTION.log'
_ERROR_HANDLE = logging.FileHandler(_ERROR_LOG_DIR)
# |=-=-=-=-=-=-=-=-=-=-=-=-=-=-=|


@dataclass
class LEVEL:
    """This Dataclass Defines The Custom Logging Level"""
    N: level = logging.NOTSET
    """Logging Level 'NOTSET'"""
    D: level = logging.DEBUG
    """Logging Level 'DEBUG'"""
    I: level = logging.INFO
    """Logging Level 'INFO'"""
    W: level = logging.WARNING
    """Logging Level 'WARNING'"""
    E: level = logging.ERROR
    """Logging Level 'ERROR'"""
    C: level = logging.CRITICAL
    """Logging Level 'CRITICAL'"""


@dataclass(frozen=True)
class MODE:
    """This Dataclass Defines The Handle Mode"""
    F: mode = frozenset('file')
    """Logging Output to File"""
    S: mode = frozenset('stream')
    """Logging Output to Console"""


class Log_Out:
    def __init__(
        self,
        output_dir_path: str = None,
        output_logname: str = None,
        handle_mode: MODE = MODE.F
    ):
        """handle_mode('file' or not set) = log data is input to file\n
        'stream' = log data is output to console"""
        if output_dir_path is None:
            output_dir_path = _DEFAULT_PATH
        if output_logname is None:
            output_logname = _DEFAULT_LOGNAME
        self._DIRECTORY: str = op.join(output_dir_path, output_logname)

        if handle_mode is MODE.F:
            self._HANDLEMODE = logging.FileHandler(self._DIRECTORY)
        elif handle_mode is MODE.S:
            self._HANDLEMODE = logging.StreamHandler()

    def __enter__(self):
        # create a logger preset
        self.normal_debug_logger = (
            logging.getLogger(f'normal_debug_log[{_PARENT_NAME}]')
        )
        self.normal_debug_logger.setLevel(logging.DEBUG)
        self.valiable_debug_logger = (
            logging.getLogger(f'look_valiable_content[{_PARENT_NAME}]')
        )
        self.valiable_debug_logger.setLevel(logging.DEBUG)

        # |=-=-=-=|Special Logger|=-=-=-=|
        logformat = logging.Formatter(
            fmt=(
                '''%(asctime)s: %(name)s [ %(lineno)d ] \
                {\n\t%(message)s\n}\n'''
            ),
            datefmt='%y%m%d%H%M%S'
        )
        self.exception_endpattern = (
            logging.getLogger('!!!EXCEPTION LOG!!!')
        )
        self.exception_endpattern.setLevel(logging.ERROR)
        _ERROR_HANDLE.setFormatter(logformat)
        self.exception_endpattern.addHandler(_ERROR_HANDLE)
        # |=-=-=-=-=-=-=-=-=-=-=-=-=-=-=|
        return self

    def val(
        self,
        log_out_content: str,
        custom_loglevel: LEVEL = None
    ) -> None:
        """\
            The custom_loglevel should be
             set using the class 'LEVEL' variable
        """
        if custom_loglevel is not None:
            self.valiable_debug_logger.setLevel(custom_loglevel)

        logformat = logging.Formatter(
            fmt=(
                '''%(asctime)s: %(levelname)s - %(name)s {\t%(message)s\t}'''
            ),
            datefmt='%y%m%d%H%M%S'
        )
        self._HANDLEMODE.setFormatter(logformat)
        self.valiable_debug_logger.addHandler(self._HANDLEMODE)

        self.valiable_debug_logger.debug(f'value: {log_out_content}')

    def flog(
        self,
        custom_loglevel: LEVEL = None,
        **kwargs,
    ) -> None:
        """The custom_loglevel should be \
        set using the class 'LEVEL' variable"""
        if custom_loglevel is not None:
            self.valiable_debug_logger.setLevel(custom_loglevel)

        logcontent = ''
        for key, value in kwargs.items():
            logcontent += f'\t{key}: {value}\n'

        logformat = logging.Formatter(
            fmt=(
                '''%(asctime)s: %(levelname)s - %(name)s {\n%(message)s}'''
            ),
            datefmt='%y%m%d%H%M%S'
        )
        self._HANDLEMODE.setFormatter(logformat)
        self.valiable_debug_logger.addHandler(self._HANDLEMODE)

        self.valiable_debug_logger.debug(logcontent)

    def __exit__(self, exc_type, exc_val, exc_tb):
        if any(val is not None for val in [exc_type, exc_val, exc_tb]):
            logging.error(
                msg='Exception Exit Log: ',
                exc_info=True,
            )
        self._HANDLEMODE.close()
        _ERROR_HANDLE.close()


if __name__ == '__main__':
    with Log_Out(
        r"E:\DevelopmentEnvironment\python\logs",
        'log_module_sample.log',
    ) as log:
        a = 0
        b = 'val:b'
        c = __name__
        log.flog(a=a, b=b, c=c)
        a = 1 + 2 + 3 + 4 + 5 * 0
        b = 'val:after b'
        log.flog(a=a, b=b, c=c)



"Event constants for tkinter.ttk"
from dataclasses import dataclass
from typing import Literal

__all__ = [
    'Combobox',
]


@dataclass(frozen=True)
class ComboboxWidget:
    ChangeValueByList: Literal['<<ComboboxSelected>>'] = '<<ComboboxSelected>>'
    "An event when a value is changed from the dropdown list."
    ItemSelected: Literal['<<ComboboxSelected>>'] = '<<ComboboxSelected>>'
    "An event when a value is changed from the dropdown list."
    SelectionChange: Literal['<<ComboboxSelected>>'] = '<<ComboboxSelected>>'
    "An event when a value is changed from the dropdown list."


Combobox = ComboboxWidget()



"""
Provides a convenient Decorator Function for Debugging!
"""

# region	|=-=-=-=|Import  Section|=-=-=-=|
import functools
import datetime as dt
import logging as log
import os
opj = os.path.join
# endregion
# 			|=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=|

# region	|=-=-=-=|Setup Section|=-=-=-=|
__LOGGER__ = r'log.log'
# endregion
# 			|=-=-=-=-=-=-=-=-=-=-=-=-=-=-=|


def debugging(export_dir: str = None, console: bool = True):
    if not console:
        if export_dir is None:
            log_mode = log.FileHandler(__LOGGER__)
        else:
            log_mode = log.FileHandler(opj(export_dir, __LOGGER__))
    else:
        log_mode = log.StreamHandler()

    dbgLog = log.getLogger('debugging')
    dbgLog.setLevel(log.INFO)
    dbgLog_handler = log_mode
    formatter = log.Formatter(
        '%(asctime)s [%(levelname)s]: Decorator => %(name)s >>> %(message)s'
    )
    dbgLog_handler.setFormatter(formatter)
    dbgLog.addHandler(dbgLog_handler)

    def decorator(func):
        func_name = func.__name__

        @functools.wraps(func)
        def wrapper(*args, **kwargs):
            try:
                result = func(*args, **kwargs)
                dbgLog.info(
                    f"[Success Log]: function => \
{{{func_name}(*args{args}, **kwargs{kwargs})}}: \
returned -> {result}"
                )
            except Exception as e:
                dbgLog.error(
                    f"[Error Log]: function => \
{{{func_name}(*args{args}, **kwargs{kwargs})}}: {str(e)}"
                )
                result = None

            return result

        return wrapper

    return decorator


def process_time(func):
    ptLog = log.getLogger('process_time')
    ptLog.setLevel(log.INFO)
    ptLog_handler = log.StreamHandler()
    formatter = log.Formatter(
        '%(asctime)s [%(levelname)s]: Decorator => %(name)s >>> %(message)s'
    )
    ptLog_handler.setFormatter(formatter)
    ptLog.addHandler(ptLog_handler)

    @functools.wraps(func)
    def wrapper(*args, **kwargs):
        start = dt.datetime.now()
        result = func(*args, **kwargs)
        ptLog.info(f"[ProcessTime Info]: function => \
{{{func.__name__}(*args{args}, **kwargs{kwargs})}} -> \
{(dt.datetime.now() - start).total_seconds()}s")

        return result

    return wrapper


if __name__ == '__main__':
    logfile = r"E:\DevelopmentEnvironment\ERROR LOG"

    @process_time
    @debugging(logfile, False)
    def test(a, b):
        return a / b

    print(test(9, 4))



from dataclasses import dataclass
from typing import Literal

__all__ = [
    'Listbox',
    'Text',
    'Tk',
    'Toplevel',
]


@dataclass(frozen=True)
class Window:
    Resize: Literal["<Configure>"] = "<Configure>"
    """Example:
        my_window.bind(Resize, lambda event: print(event))"""


@dataclass(frozen=True)
class TextWidget:
    PressEnter: Literal["<Return>"] = "<Return>"
    "Events when the Enter key is pressed"
    AnyKey: Literal["<Key>"] = "<Key>"
    "Events when any key is pressed"


@dataclass(frozen=True)
class ListWidget:
    ItemSelect: Literal['<<ListboxSelect>>'] = '<<ListboxSelect>>'
    "Events when the any item is selected"


Listbox = ListWidget()
Text = TextWidget()
Tk = Window()
Toplevel = Window()



from copy import deepcopy
import tkinter as tk
from tkinter import font
from typing import Any, Iterable, Self, Sized
AUTO = -1


class Counter:
    def __init__(self, *, item: Sized | None = None, min_max: tuple[int, int] = (0, 0), start_count: int = 0, loopable: bool = False) -> None:
        if item is None:
            self.min = min_max[0]
            self.max = min_max[1]
        else:
            self.min = 0
            self.max = len(item) - 1
        self._loopable = loopable
        self._current = start_count if start_count >= self.min and start_count <= self.max else self.min

    def __str__(self):
        return str(self._current)

    def __repr__(self):
        return f'{self._current!r}'

    def __int__(self):
        return self._current

    def __add__(self, value: int) -> 'Counter':
        if self.ulimit:
            return self
        else:
            aself = deepcopy(self)
            if aself.over_ulimit(value):
                if aself._loopable:
                    aself._current = (aself._current + value - aself.max - 1)
                else:
                    aself._current = aself.max
            else:
                aself._current += value
            return aself

    def __iadd__(self, value: int) -> Self:
        if self.ulimit:
            return self
        else:
            if self.over_ulimit(value):
                if self._loopable:
                    self._current = (self._current + value - self.max - 1)
                else:
                    self._current = self.max
            else:
                self._current += value
            return self

    def __sub__(self, value: int) -> 'Counter':
        if self.llimit:
            return self
        else:
            aself = deepcopy(self)
            if aself.over_llimit(value):
                if aself._loopable:
                    aself._current = aself.max + (aself._current - value - aself.min + 1)
                else:
                    aself._current = aself.min
            else:
                aself._current -= value
            return aself

    def __isub__(self, value: int) -> Self:
        if self.llimit:
            return self
        else:
            if self.over_llimit(value):
                if self._loopable:
                    self._current = self.max + (self._current - value - self.min + 1)
                else:
                    self._current = self.min
            else:
                self._current -= value
            return self

    def __index__(self) -> int:
        return self._current

    def over_ulimit(self, value: int) -> bool:
        return self.max < self._current + value

    def over_llimit(self, value: int) -> bool:
        return self.min > self._current - value

    def inc(self):
        self += 1

    def dec(self):
        self -= 1

    @property
    def ulimit(self) -> bool:
        return self._current == self.max and not self._loopable

    @property
    def llimit(self) -> bool:
        return self._current == self.min and not self._loopable


class Transition(tk.Frame):
    def __init__(self, master: tk.Misc | None = None, *arg, loop: bool = False, **kw) -> None:
        super().__init__(master, *arg, **kw)
        self._widgets: list[tk.Widget] = []
        self.current: Counter | None = None
        self._packing: tuple[tuple[Any, ...], dict[str, Any]] | None = None
        self._loop = loop

    def __getitem__(self, id: int):
        return self._widgets[id]

    def __add__(self, value: tk.Widget | Iterable[tk.Widget]):
        aself = deepcopy(self)
        aself.register_widget(value)
        return aself

    def __iadd__(self, value: tk.Widget | Iterable[tk.Widget]):
        self.register_widget(value)
        return self

    def register_widget(self, widget: tk.Widget | Iterable[tk.Widget], *, id: int | slice = AUTO):
        if isinstance(id, slice):
            if any(n >= len(self._widgets) for n in [id.start, id.stop]):
                raise IndexError('不正なID指定です。')
            else:
                item = [widget] if not isinstance(widget, Iterable) else widget
                self._widgets[id] = item
        elif id >= 0:
            if id >= len(self._widgets):
                raise IndexError('不正なID指定です。')
            elif isinstance(widget, Iterable):
                key = slice(id, id)
            else:
                key = id
            self._widgets[key] = widget
        else:
            self._widgets += widget if isinstance(widget, Iterable) else [widget]

    def prepack(self, *arg: Any, **kw: Any):
        self._packing = (arg, kw)

    def initial_pack(self, id: int = 0):
        if self._packing is None:
            raise ValueError('パックオプションが未設定です。\nprepackメソッドでパックオプションをあらかじめ設定してください。')
        self._widgets[id].pack(*self._packing[0], **self._packing[1])
        self.current = Counter(item=self._widgets, loopable=self._loop, start_count=id)

    def next(self):
        if self.current is None:
            self.current = Counter(item=self._widgets, loopable=self._loop)
        elif self.current.ulimit:
            return True
        else:
            self._widgets[self.current].pack_forget()
            self.current.inc()
        self._widgets[self.current].pack(*self._packing[0], **self._packing[1])
        return False

    def prev(self):
        if self.current is None:
            self.current = Counter(item=self._widgets, loopable=self._loop)
        elif self.current.llimit:
            return True
        else:
            self._widgets[self.current].pack_forget()
            self.current.dec()
        self._widgets[self.current].pack(*self._packing[0], **self._packing[1])
        return False


def sample(loop: bool = True, widget_number: int = 3):
    root = tk.Tk()
    root.geometry('250x250')
    trans = Transition(root, loop=loop)
    trans.pack(side=tk.TOP, anchor=tk.S, fill=tk.BOTH, expand=True)
    trans.prepack(fill=tk.BOTH, expand=True)

    for i in range(1, widget_number + 1):
        _tmp_ = tk.Frame(trans)
        tk.Label(_tmp_, text=f'LABEL{i}', font=font.Font(size=20)).pack(fill=tk.BOTH, expand=True)
        trans.register_widget(_tmp_)

    bf = tk.Frame(root)
    bf.pack(side=tk.TOP, anchor=tk.S, fill=tk.X, expand=True)
    tk.Button(bf, text='NEXT', command=trans.next).pack(side=tk.RIGHT, anchor=tk.E)
    tk.Button(bf, text='PREV', command=trans.prev).pack(side=tk.RIGHT, anchor=tk.E, padx=5)
    trans.initial_pack()
    root.mainloop()


if __name__ == '__main__':
    sample(widget_number=5)



"""~~~python

from sqlalchemy import (
    Column,
    create_engine,
    Engine,
    ForeignKey,
    MetaData,
    Table,
    text,
)
from sqlalchemy.types import (
    Boolean,
    Integer,
    LargeBinary,
    String,
    Double,
    Float,
    BigInteger,
    SmallInteger,
    Time,
    Date,
    DateTime,
    Enum,
    Interval,
    MatchType,
    Numeric,
    PickleType,
    Text,
    Unicode,
    UnicodeText,
)
from sqlalchemy.orm import (
    declarative_base,
    DeclarativeMeta,
    scoped_session,
    Session,
    sessionmaker,
)
"""
from sys import modules as __modules
from sqlalchemy import (
    Column,
    create_engine,
    Engine,
    ForeignKey,
    MetaData,
    Table,
    text,
)
from sqlalchemy.types import (
    Boolean,
    Integer,
    LargeBinary,
    String,
    Double,
    Float,
    BigInteger,
    SmallInteger,
    Time,
    Date,
    DateTime,
    Enum,
    Interval,
    MatchType,
    Numeric,
    PickleType,
    Text,
    Unicode,
    UnicodeText,
)
from sqlalchemy.orm import (
    declarative_base,
    DeclarativeMeta,
    scoped_session,
    Session,
    sessionmaker,
)

__root__ = {k: v.__module__ for k, v in __modules[__name__].__dict__.copy().items() if not k.startswith('__')}
__imported__ = {k: v for k, v in __modules[__name__].__dict__.copy().items() if not k.startswith('__')}



"""
The functions in this module can be used in "if statements"
"""
from typing import (
    Iterable,
    Any
)
__all__ = [
    'excepting',
]


def excepting(other_than: Any, judg_target: Iterable) -> bool:
    return any(item != other_than for item in judg_target)



from console import clear as cls
flg = True
mode = None
while flg:
    flg = False
    cls()
    print('1- A : B = C : x')
    mode = int(input('mode select:\n'))
    if mode == '':
        flg = True
cls()
if mode == 1:
    print('Mode Number Please')
    print('A : B = C : x')
    a = int(input('A = ?\n'))
    b = int(input('B = ?\n'))
    c = int(input('C = ?\n'))
    print('Solve and Answer')
    print(f'\t{a}x = {b}・{c}')
    print(f'\t{a}x = {b * c}')
    print(f'\t\033[92mx = {(b * c) / a}\033[0m')



"""
SQL databases controler
"""
from typing import Any, Callable
from sqlalchemy import BinaryExpression, Column, ColumnElement, create_engine, MetaData, Table, Engine as _Engine
from sqlalchemy.orm import sessionmaker, Session as _Session, DeclarativeMeta


def items(url_or_engine: str | _Engine = 'sqlite:///:memory:', *, print_to_console=True, **kw):
    # データベースエンジンの設定
    engine = ...
    if isinstance(url_or_engine, str):
        engine = create_engine(url_or_engine, **kw)
    elif isinstance(url_or_engine, _Engine):
        engine = url_or_engine
    else:
        return {}
    Session = sessionmaker(bind=engine)
    session = Session()

    # メタデータの読み込み
    metadata = MetaData()
    metadata.reflect(bind=engine)
    result = {}
    for table_name in metadata.tables:
        result[table_name] = []
        table = Table(table_name, metadata, autoload_with=engine)
        if print_to_console: print(f"--- {table_name} ---")
        # 各テーブルからすべてのデータを取得して表示
        for row in session.query(table).all():
            result[table_name].append(row)
            if print_to_console: print(row)
    return result


def query_gen(session: _Session, table: DeclarativeMeta | None = None,
              *filters: ColumnElement[bool] | BinaryExpression[bool],
              delete=False, commit=False):
    if not delete:
        it = ...
        if filters:
           it = session.query(table).filter(*filters).all
        else:
           it = session.query(table).all
        for item in it():
            yield item
    else:
        def _f():
            result = session.query(table).filter(*filters).first() if filters else session.query(table).first()
            if result:
                session.delete(result)
                if commit:
                    session.commit()
            return result
        for item in iter(_f, None):
            yield item



# region	|=-=-=-=|Import  Section|=-=-=-=|
from collections import namedtuple
import re
# endregion
# 			|=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=|

# region	|=-=-=-=|Setup Section|=-=-=-=|
__all__ = [
    'INIsNotFound',
    'INI',
    'AbsTrue',
    'AbsFalse',
    'AbsTrueFalse',
    'StrBool',
    'str_bool'
]
Encoding = namedtuple('Encode', ['SJIS', 'UTF8'])('shift-jis', 'utf-8')
# endregion
# 			|=-=-=-=-=-=-=-=-=-=-=-=-=-=-=|


# iniファイルない時用の例外
class INIsNotFound(Exception):
    # .iniファイル必要なのに無いやで。ってやつ

    def __str__(self):
        return '404 Not Found'


class INI:
    'キーダブり対応版'
    def __init__(self, filepath=None, encoding=Encoding.SJIS):
        'キーダブり対応版'
        # filepathがNoneじゃないなら
        if filepath is not None:
            # config管理用の箱用意
            self.config = namedtuple('Config', ['export'])(filepath)
            # 属性file_pathとconfigのexport項目 = filepath
            self.file_path = filepath
        # エンコードキー
        self.chrdec = encoding
        # コンテキストフラグ初期化
        self._context = False
        # .iniの内容入れる用
        self.ini_content = {}
        # ダブりセクションキー: 一時リネームした個数
        self._already_exist = {}
        # 一時リネームしたキー名: 元のキー名
        self._exist_lower = {}

    def _keys(self, ini_contents: dict):
        # 例外回避
        assert isinstance(ini_contents, dict), '正しいアイテムを渡してください。'
        # 渡されたdictのキー取得＆リスト化して返す
        return [k for k in ini_contents.keys()]

    def _ini_item(self, item, value):
        return f'{item}={value}'

    def _item(self, sect, items):
        name, value = items
        section = self.isexist(sect)
        self.ini_content[section][name] = value if str_bool(
            value) is None else str_bool(value)

    def _section(self, name):
        # nameがself.ini_contentに既に存在してるなら
        if name in self.ini_content:
            # self._already_existに既に存在してるなら
            if name in self._already_exist:
                # ダブりでリネームした個数に + 1
                self._already_exist[name] += 1
            else:
                # ダブりでリネームした個数 = 1で新規追加
                self._already_exist[name] = 1

        # 既に存在してるならsection = リネーム名, 否でsection = name
        section = self.isexist(name)
        # self.ini_contentのsectionの項目を新規作成
        self.ini_content[section] = {}

    def _section_wrapp(self, key):
        return f'[{key}]'

    def _write_section(self, section, wrapp=False):
        result = \
            self._exist_lower[section] if section in self._exist_lower \
            else section
        return self._section_wrapp(result) if wrapp else result

    def export(self):
        with open(self.config.export, 'w') as f:
            sections = self._keys(self.ini_content)
            for section in sections:
                f.write(f'{self._write_section(section, wrapp=True)}\n')
                item_dict = self.ini_content[section]
                items = self._keys(item_dict)
                for item in items:
                    f.write(f'{self._ini_item(item, item_dict[item])}\n')

                f.write('\n')

    def isexist(self, key):
        if key in self._already_exist:
            result = f'{key}{self._already_exist[key]}'
            self._exist_lower[result] = key
        else:
            result = key

        return result

    def read(self, ini_file=None):
        current_section = None
        if self._context:
            content = self.file
        elif ini_file is not None:
            content = ini_file
        else:
            raise INIsNotFound

        for cont in content:
            if cont.startswith('['):
                sect = re.sub(r'\[|\]|\n', '', cont)
                self._section(sect)
                current_section = sect
            elif '=' in cont:
                item = cont.split('=')
                for i in range(len(item)):
                    item[i] = re.sub(
                        r'\n', '', item[i]
                    ) if '\n' in item[i] else item[i]
                self._item(current_section, (item[0], item[1]))

        return self.ini_content

    def __getitem__(self, key):
        if len(key) == 2:
            section, item = key
            if section in self.ini_content:
                if item in self.ini_content[section]:
                    return self.ini_content[section][item]
                else:
                    return self.ini_content[section]
            else:
                return None
        elif key == '*':
            return self.ini_content
        else:
            if key in self.ini_content:
                return self.ini_content[key]
            else:
                return None

    def __setitem__(self, key, value):
        if len(key) == 2:
            section, item = key
            if section in self.ini_content:
                if item in self.ini_content[section]:
                    self.ini_content[section][item] = value
                elif isinstance(value, dict):
                    itemkey, itemvalue = value
                    self.ini_content[section][itemkey] = itemvalue
                else:
                    self.ini_content[section][item] = value
        else:
            if key in self.ini_content:
                self.ini_content[key][value] = {}

    def __enter__(self):
        self._context = True
        self.file = open(self.file_path, encoding=self.chrdec)
        self.read()

        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.file.close()


class _StrOrBool:
    def __init__(self, content: '_StrOrBool'):
        self.content = content

    def STRING(self, content: '_StrOrBool' = None):
        ...

    def BOOL(self, content: '_StrOrBool' = None):
        ...

    def __str__(self):
        """
        Example:
            _SubStrOrBool_instance << str('true')
            >>> print(str(_SubStrOrBool_instance))
            bool(True)
        """
        return self.STRING(self.content)

    def __init_true__(self):
        class _true:
            def __init__(subself):
                subself.trues = (True, 1, 'true', '1')

            @property
            def true(subself):
                return subself.trues

            def __contain__(subself, item: str | bool | int):
                if isinstance(item, str):
                    return item.lower() in subself.trues
                else:
                    return item in subself.trues

        return _true()

    def __init_false__(self):
        class _false:
            def __init__(subself):
                subself.falses = (False, 0, 'false', '0', None)

            @property
            def false(subself):
                return subself.falses

            def __contain__(subself, item: str | bool | int):
                if isinstance(item, str):
                    return item.lower() in subself.falses
                else:
                    return item in subself.falses

        return _false()

    def __init_subclass__(cls, *args, **kwargs) -> None:
        super().__init_subclass__(*args, **kwargs)
        cls.true = cls.__init_true__(cls)
        cls.false = cls.__init_false__(cls)

    @property
    def trues(self):
        return self.__init_true__().trues

    @property
    def falses(self):
        return self.__init_false__().falses


class AbsTrue(_StrOrBool):
    def __init__(self, content: _StrOrBool):
        super().__init__(content)

    def __bool__(self) -> bool:
        return self.content in self.true


class AbsFalse(_StrOrBool):
    def __init__(self, content: _StrOrBool):
        super().__init__(content)

    def __bool__(self) -> bool:
        return self.content in self.false


class AbsTrueFalse(_StrOrBool):
    def __init__(self, content: _StrOrBool):
        super().__init__(content)

    def __bool__(self) -> bool:
        if self.content in self.true:
            return True
        elif self.content in self.false:
            return False
        else:
            return None


class StrBool:
    def __new__(cls, content: _StrOrBool, absolute: bool = None):
        if absolute is True:
            self: AbsTrue = AbsTrue(content)
        elif absolute is False:
            self: AbsFalse = AbsFalse(content)
        else:
            self: AbsTrueFalse = AbsTrueFalse(content)

        return self


def str_bool(content: str):
    trues = (True, 1, 'true', '1')
    falses = (False, 0, 'false', '0', None)

    if content.lower() in trues:
        return True
    elif content.lower() in falses:
        return False
    else:
        return None


if __name__ == '__main__':
    print(Encoding)



from typing import Any, ParamSpec, Self, TextIO, Callable, TypeVar
import sys

__all__ = [
    "AccessObserver",
    "StdIOTestSpace",
    "CallFollowerMeta",
]


class AccessObserver:
    def __init__(self, name: str, target_object: object, o_out: TextIO = sys.stdout, chain_observe: bool = True) -> None:
        self.name = name
        self.target = target_object
        self.stdout = o_out
        self.chain = chain_observe

    def __getattr__(self, name: str):
        if self.chain:
            def args_viewer(*args: Any, **kwargs: Any):
                result = None
                if name == 'fileno':
                    result = 0
                elif name == 'readline':
                    result = 'test line'
                print(f'{self.name}\nAccessName: {name}\nArgs: {args}\nKwArgs: {kwargs}', file=self.stdout)
                result = self if result is None else result
                return result
            return args_viewer
        else:
            print(f'AccessName: {name}')
            return getattr(self.target, name)


class StdIOTestSpace:
    def __enter__(self) -> Self:
        self.o_in = sys.stdin
        self.o_out = sys.stdout
        self.o_err = sys.stderr
        sys.stdin = AccessObserver('sys.stdin', self.o_out)
        sys.stdout = AccessObserver('sys.stdout', self.o_out)
        sys.stderr = AccessObserver('sys.stderr', self.o_out)
        return self

    def __exit__(self, exc_type: Any, exc_val: Any, exc_tb: Any) -> bool:
        sys.stdin = self.o_in
        sys.stdout = self.o_out
        sys.stderr = self.o_err
        return False


_P = ParamSpec('_P')
_R = TypeVar('_R')


def printer(func: Callable[_P, _R]) -> Callable[_P, _R]:
    def wrapper(*arg: _P.args, **kw: _P.kwargs) -> _R:
        try:
            print(f'CALL: {func.__qualname__}')
        except AttributeError:
            print(f'CALL: Unknown.{func.__name__}')
        return func(*arg, **kw)
    return wrapper


def wrapping_machine(dic: dict[str, Any]) -> dict[str, Any]:
    for k, v in object.__dict__.items():
        if k not in dic and v is not object.__new__:
            dic[k] = v
    for k, v in dic.items():
        dic[k] = printer(v) if callable(v) else v
    return dic


class CallFollowerMeta(type):
    """Usage:
```python
class MyClass(metaclass=CallFollowerMeta):
    def __init__(self, value):
        self.value = value

instance = MyClass('string')
# CALL: MyClass.__init__
# CALL: object.__setattr__
print(instance)
# CALL: object.__str__
# CALL: object.__repr__
```"""
    def __new__(cls, name: str, bases: tuple[type, ...], dic: dict[str, Any]):
        return super().__new__(cls, name, bases, wrapping_machine(dic))



from setuptools import setup, find_packages

setup(
    name='memodules',
    version='5.10.2',
    author='mie31',
    author_email='mie.mey.master@icloud.com',
    description='local use functions',
    long_description="We do not consider the use of it by third parties.",
    long_description_content_type="text/plain",
    license="MIT",
    keywords='No redistribution!',
    packages=find_packages(),  # インポートするパッケージのリスト
    package_data={
        '': ['*.pyi'],
    },
    python_requires=">=3.11",
    classifiers=[
        'Intended Audience :: Developers',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.11',
    ],
)



import re

def AtI(alphabet: str) -> int:
    """A(or a) -> 1\n
    AA(or aa) -> 27"""
    result = 0
    if type(alphabet) == list:
        result = []
        for i in alphabet:
            mid = 0
            girder = len(i) - 1
            for n in i:
                mid += (ord(n.upper()) - ord('A') + 1) * (26 ** girder)
                girder -= 1
            
            result.append(mid)
    elif type(alphabet) == str:
        girder = len(alphabet) - 1
        for i in alphabet:
            result += (ord(i.upper()) - ord('A') + 1) * (26 ** girder)
            girder -= 1
        
    return result

def ItA(integer: int, large: bool=False) -> str:
    """integer = 1, large = False -> a\n
    integer = 27, large = True -> AA"""
    result = ''
    tgt = integer
    find = False
    g = 0
    mid = 0
    girder = 0
    while find == False:
        if g == 0:
            if integer <= 1:
                find = True
                girder = 1
        else:
            for i in range(0, g+1):
                mid += (26 ** i)
            if integer < mid:
                find = True
                girder = g
            else:
                mid = 0
                
        g += 1
        
    alp = []
    for gi in range(girder-1, -1, -1):
        for i in range(1, 27):
            if tgt < (26 ** gi) * (i + 1):
                alp.append(i)
                tgt -= (26 ** gi) * i
                break
            elif i == 26:
                alp.append(26)
                tgt -= (26 ** gi) * 26
            
    for i in alp:
        result += chr(i + ord('A') - 1)
    
    return result.lower() if large == False else result.upper()

def pickalpha(target):
    """example: 
    pickalpha('A1') -> 'A'
    pickalpha(['A1', 'AT1']) -> ['A', 'AT']"""
    result = ''
    if type(target) == list:
        result = [''] * len(target)
        i = 0
        for s in target:
            for ss in s:
                if ss.isalpha():
                    result[i] += ss
                    
            i += 1
    elif type(target) == str:
        for s in target:
            if s.isalpha():
                result += s
                
    return result

def SIsplit(target: str) -> list[str, str]:
    result = ['', '']
    for s in target:
        if s.isalpha():
            result[0] += s
        else:
            result[1] += s
            
    return result

def alphain(string):
    pattern = r'[A-Za-z]+'
    match = re.search(pattern, string)
    return match is not None
    
if __name__ == '__main__':
    print(ItA(27, False))
elif __name__ == '____':
    print(ord('Z'))
    print(chr(91))
elif __name__ == '____':
    a = 'X20'
    print(SIsplit(a))
elif __name__ == '____':
    a = ['X20', 'AT20']
    r = AtI(pickalpha(a))
    print(r)
elif __name__ == '____':
    print(f'ati("AAA"): {AtI("AAA")}')
    print(f'pickalpha("X20"): {pickalpha("X20")}')
    print(f'pickalpha(["X20", "AT20"]): {pickalpha(["X20", "AT20"])}')



from memodules.alphabet_to_integer import *
from memodules.error_lib import *
from dataclasses import dataclass
import re

class OVER_RANGE(Exception):
    """Max Range of Excel Application:\nMaxRows = 1048576, MaxColumn = 16384\n
    When it exceeds that number, an 'OVER_RANGE' error occurs"""
    def __init__(self, mode: str):
        self.mode = mode
    
    def __str__(self) -> str:
        return f'{self.mode} Range Over'
    
def isoverrange(target: int, mode: str, ifmode: bool=False) -> bool:
    """mode = 'r': target > 1048576 -> OVER_RANGE Error\n
    mode = 'c': target > 16384 -> OVER_RANGE Error"""
    if mode.lower().startswith('r'):
        if target > xl_Data_Dict._MAX_ROW:
            if ifmode:
                return True
            else:
                raise OVER_RANGE(mode='Rows')
    elif mode.lower().startswith('c'):
        if target > xl_Data_Dict._MAX_COLUMN:
            if ifmode:
                return True
            else:
                raise OVER_RANGE(mode='Columns')
    else:
        modeselectionerror()

@dataclass(frozen=True)
class xl_Data_Dict:
    """xl_Data_Dict.varlist()で定数リスト取得可"""
    _MAX_ROW: int = 1048576
    """The Bottom Value of an Excel Application Worksheet"""
    _MAX_COLUMN: int = 16384
    """The Rightmost Value of an Excel Application Worksheet"""
    _MAX_COLUMN_LETTER: str = frozenset('XFD')
    """The Rightmost Index Letter an Excel Application Worksheet"""
    
    @classmethod
    def varlist(cls):
        initialized_variables = []
        for name, value in cls.__dict__.items():
            if not name.startswith("__") and not callable(value):
                if value == getattr(cls, name):
                    initialized_variables.append(name)
        return initialized_variables

def cell_index_increment(target: str, incrementvalue: int=1, mode: str='r'):
    """target = 'C7'\n
    mode = 'r': -> 'C8'\n
    mode = 'c': -> 'D7'"""
    cidx, ridx = SIsplit(target)
    result = ''
    if mode.lower().startswith('r'):
        ridx = int(ridx) + incrementvalue
        result = f'{cidx}{ridx}'
    elif mode.lower().startswith('c'):
        cidx = ItA(AtI(cidx) + incrementvalue, True)
        result = f'{cidx}{ridx}'
        
    return result
                
def range_index_exchange(target, mode: str):
    """mode = 'range': 'C7' ->tuple (7, 3)(int, int)\n
    mode = 'cells': [7, 3] or (7, 3) ->str 'C7'"""
    #mode分岐
    row = ''
    column = ''
    if mode == 'range':
        column, row = SIsplit(target)
        column = AtI(column)
        return (int(row), column)
    elif mode == 'cell':
        row, column = target
        column = ItA(column, True)
        return f'{column}{row}'
    else:
        modeselectionerror()
        
if __name__ == '__main__':
    print(xl_Data_Dict)